@@ -12,6 +12,7 @@ extern struct obs_source_info color_filter;
 extern struct obs_source_info scale_filter;
 extern struct obs_source_info scroll_filter;
 extern struct obs_source_info color_key_filter;
+extern struct obs_source_info color_grade_filter;
 extern struct obs_source_info sharpness_filter;
 extern struct obs_source_info chroma_key_filter;
 extern struct obs_source_info async_delay_filter;
@@ -19,6 +20,7 @@ extern struct obs_source_info async_delay_filter;
 extern struct obs_source_info noise_suppress_filter;
 #endif
 extern struct obs_source_info noise_gate_filter;
+extern struct obs_source_info compressor_filter;
 
 bool obs_module_load(void)
 {
@@ -29,6 +31,7 @@ bool obs_module_load(void)
 	obs_register_source(&scale_filter);
 	obs_register_source(&scroll_filter);
 	obs_register_source(&color_key_filter);
+	obs_register_source(&color_grade_filter);
 	obs_register_source(&sharpness_filter);
 	obs_register_source(&chroma_key_filter);
 	obs_register_source(&async_delay_filter);
@@ -36,5 +39,6 @@ bool obs_module_load(void)
 	obs_register_source(&noise_suppress_filter);
 #endif
 	obs_register_source(&noise_gate_filter);
+	obs_register_source(&compressor_filter);
 	return true;
 }
