@@ -37,10 +37,13 @@ using namespace DShow;
 #define LAST_VIDEO_DEV_ID "last_video_device_id"
 #define LAST_RESOLUTION   "last_resolution"
 #define BUFFERING_VAL     "buffering"
+#define FLIP_IMAGE        "flip_vertically"
+#define AUDIO_OUTPUT_MODE "audio_output_mode"
 #define USE_CUSTOM_AUDIO  "use_custom_audio_device"
 #define AUDIO_DEVICE_ID   "audio_device_id"
 #define COLOR_SPACE       "color_space"
 #define COLOR_RANGE       "color_range"
+#define DEACTIVATE_WNS    "deactivate_when_not_showing"
 
 #define TEXT_INPUT_NAME     obs_module_text("VideoCaptureDevice")
 #define TEXT_DEVICE         obs_module_text("Device")
@@ -58,6 +61,11 @@ using namespace DShow;
 #define TEXT_BUFFERING_AUTO obs_module_text("Buffering.AutoDetect")
 #define TEXT_BUFFERING_ON   obs_module_text("Buffering.Enable")
 #define TEXT_BUFFERING_OFF  obs_module_text("Buffering.Disable")
+#define TEXT_FLIP_IMAGE     obs_module_text("FlipVertically")
+#define TEXT_AUDIO_MODE     obs_module_text("AudioOutputMode")
+#define TEXT_MODE_CAPTURE   obs_module_text("AudioOutputMode.Capture")
+#define TEXT_MODE_DSOUND    obs_module_text("AudioOutputMode.DirectSound")
+#define TEXT_MODE_WAVEOUT   obs_module_text("AudioOutputMode.WaveOut")
 #define TEXT_CUSTOM_AUDIO   obs_module_text("UseCustomAudioDevice")
 #define TEXT_AUDIO_DEVICE   obs_module_text("AudioDevice")
 #define TEXT_ACTIVATE       obs_module_text("Activate")
@@ -67,6 +75,7 @@ using namespace DShow;
 #define TEXT_COLOR_RANGE    obs_module_text("ColorRange")
 #define TEXT_RANGE_PARTIAL  obs_module_text("ColorRange.Partial")
 #define TEXT_RANGE_FULL     obs_module_text("ColorRange.Full")
+#define TEXT_DWNS           obs_module_text("DeactivateWhenNotShowing")
 
 enum ResType {
 	ResType_Preferred,
@@ -107,6 +116,7 @@ public:
 	inline ~Decoder() {ffmpeg_decode_free(&decode);}
 
 	inline operator ffmpeg_decode*() {return &decode;}
+	inline ffmpeg_decode *operator->() {return &decode;}
 };
 
 class CriticalSection {
@@ -153,7 +163,9 @@ static DWORD CALLBACK DShowThread(LPVOID ptr);
 struct DShowInput {
 	obs_source_t *source;
 	Device       device;
+	bool         deactivateWhenNotShowing = false;
 	bool         deviceHasAudio = false;
+	bool         flip = false;
 	bool         active = false;
 
 	Decoder      audio_decoder;
@@ -366,6 +378,7 @@ static inline video_format ConvertVideoFormat(VideoFormat format)
 	case VideoFormat::I420:  return VIDEO_FORMAT_I420;
 	case VideoFormat::YV12:  return VIDEO_FORMAT_I420;
 	case VideoFormat::NV12:  return VIDEO_FORMAT_NV12;
+	case VideoFormat::Y800:  return VIDEO_FORMAT_Y800;
 	case VideoFormat::YVYU:  return VIDEO_FORMAT_YVYU;
 	case VideoFormat::YUY2:  return VIDEO_FORMAT_YUY2;
 	case VideoFormat::UYVY:  return VIDEO_FORMAT_UYVY;
@@ -407,6 +420,8 @@ void DShowInput::OnEncodedVideoData(enum AVCodecID id,
 
 	if (got_output) {
 		frame.timestamp = (uint64_t)ts * 100;
+		if (flip)
+			frame.flip = !frame.flip;
 #if LOG_ENCODED_VIDEO_TS
 		blog(LOG_DEBUG, "video ts: %llu", frame.timestamp);
 #endif
@@ -430,10 +445,12 @@ void DShowInput::OnVideoData(const VideoConfig &config,
 	frame.width      = config.cx;
 	frame.height     = config.cy;
 	frame.format     = ConvertVideoFormat(config.format);
-	frame.full_range = false;
 	frame.flip       = (config.format == VideoFormat::XRGB ||
 	                    config.format == VideoFormat::ARGB);
 
+	if (flip)
+		frame.flip = !frame.flip;
+
 	if (videoConfig.format == VideoFormat::XRGB ||
 	    videoConfig.format == VideoFormat::ARGB) {
 		frame.data[0]     = data;
@@ -468,6 +485,10 @@ void DShowInput::OnVideoData(const VideoConfig &config,
 		frame.linesize[0] = cx;
 		frame.linesize[1] = cx;
 
+	} else if (videoConfig.format == VideoFormat::Y800) {
+		frame.data[0] = data;
+		frame.linesize[0] = cx;
+
 	} else {
 		/* TODO: other formats */
 		return;
@@ -487,10 +508,18 @@ void DShowInput::OnEncodedAudioData(enum AVCodecID id,
 			blog(LOG_WARNING, "Could not initialize audio decoder");
 			return;
 		}
+
+		if (videoConfig.name.find(L"C875") != std::string::npos ||
+		    videoConfig.name.find(L"C835") != std::string::npos) {
+			audio_decoder->fix_braindead_lgp_audio_packet_stupidity
+				= true;
+			blog(LOG_INFO, "Oh great, an LGP was detected.  "
+					"How wonderful.  I'm just ecstatic.");
+		}
 	}
 
 	bool got_output;
-	int len = ffmpeg_decode_audio(audio_decoder, data, size,
+	int len = ffmpeg_decode_audio(audio_decoder, data, size, &ts,
 			&audio, &got_output);
 	if (len < 0) {
 		blog(LOG_WARNING, "Error decoding audio");
@@ -725,19 +754,29 @@ inline void DShowInput::SetupBuffering(obs_data_t *settings)
 	obs_source_set_flags(source, flags);
 }
 
+static DStr GetVideoFormatName(VideoFormat format);
+
 bool DShowInput::UpdateVideoConfig(obs_data_t *settings)
 {
 	string video_device_id = obs_data_get_string(settings, VIDEO_DEVICE_ID);
+	deactivateWhenNotShowing = obs_data_get_bool(settings, DEACTIVATE_WNS);
+	flip = obs_data_get_bool(settings, FLIP_IMAGE);
 
 	DeviceId id;
-	if (!DecodeDeviceId(id, video_device_id.c_str()))
+	if (!DecodeDeviceId(id, video_device_id.c_str())) {
+		blog(LOG_WARNING, "%s: DecodeDeviceId failed",
+			obs_source_get_name(source));
 		return false;
+	}
 
 	PropertiesData data;
 	Device::EnumVideoDevices(data.devices);
 	VideoDevice dev;
-	if (!data.GetDevice(dev, video_device_id.c_str()))
+	if (!data.GetDevice(dev, video_device_id.c_str())) {
+		blog(LOG_WARNING, "%s: data.GetDevice failed",
+			obs_source_get_name(source));
 		return false;
+	}
 
 	int resType = (int)obs_data_get_int(settings, RES_TYPE);
 	int cx = 0, cy = 0;
@@ -747,8 +786,11 @@ bool DShowInput::UpdateVideoConfig(obs_data_t *settings)
 	if (resType == ResType_Custom) {
 		bool has_autosel_val;
 		string resolution = obs_data_get_string(settings, RESOLUTION);
-		if (!ResolutionValid(resolution, cx, cy))
+		if (!ResolutionValid(resolution, cx, cy)) {
+			blog(LOG_WARNING, "%s: ResolutionValid failed",
+				obs_source_get_name(source));
 			return false;
+		}
 
 		has_autosel_val = obs_data_has_autoselect_value(settings,
 				FRAME_INTERVAL);
@@ -763,16 +805,18 @@ bool DShowInput::UpdateVideoConfig(obs_data_t *settings)
 
 		long long best_interval = numeric_limits<long long>::max();
 		bool video_format_match = false;
-		if (!CapsMatch(dev,
-			ResolutionMatcher(cx, cy),
-			VideoFormatMatcher(format, video_format_match),
-			ClosestFrameRateSelector(interval, best_interval),
-			FrameRateMatcher(interval)) && !video_format_match)
+		bool caps_match = CapsMatch(dev, ResolutionMatcher(cx, cy),
+				VideoFormatMatcher(format, video_format_match),
+				ClosestFrameRateSelector(interval, best_interval),
+				FrameRateMatcher(interval));
+
+		if (!caps_match && !video_format_match) {
+			blog(LOG_WARNING, "%s: Video format match failed",
+				obs_source_get_name(source));
 			return false;
+		}
 
 		interval = best_interval;
-		blog(LOG_INFO, "%s: Using interval %lld",
-				obs_source_get_name(source), interval);
 	}
 
 	videoConfig.name             = id.name.c_str();
@@ -793,15 +837,49 @@ bool DShowInput::UpdateVideoConfig(obs_data_t *settings)
 	if (videoConfig.internalFormat != VideoFormat::MJPEG)
 		videoConfig.format = videoConfig.internalFormat;
 
-	if (!device.SetVideoConfig(&videoConfig))
+	if (!device.SetVideoConfig(&videoConfig)) {
+		blog(LOG_WARNING, "%s: device.SetVideoConfig failed",
+			obs_source_get_name(source));
 		return false;
+	}
 
 	if (videoConfig.internalFormat == VideoFormat::MJPEG) {
 		videoConfig.format = VideoFormat::XRGB;
-		if (!device.SetVideoConfig(&videoConfig))
+		if (!device.SetVideoConfig(&videoConfig)) {
+			blog(LOG_WARNING, "%s: device.SetVideoConfig (XRGB) "
+					"failed", obs_source_get_name(source));
 			return false;
+		}
 	}
 
+	DStr formatName = GetVideoFormatName(videoConfig.internalFormat);
+
+	double fps = 0.0;
+
+	if (videoConfig.frameInterval)
+		fps = 10000000.0 / double(videoConfig.frameInterval);
+
+	BPtr<char> name_utf8;
+	BPtr<char> path_utf8;
+	os_wcs_to_utf8_ptr(videoConfig.name.c_str(), videoConfig.name.size(),
+			&name_utf8);
+	os_wcs_to_utf8_ptr(videoConfig.path.c_str(), videoConfig.path.size(),
+			&path_utf8);
+
+	blog(LOG_INFO, "---------------------------------");
+	blog(LOG_INFO, "[DShow Device: '%s'] settings updated: \n"
+			"\tvideo device: %s\n"
+			"\tvideo path: %s\n"
+			"\tresolution: %dx%d\n"
+			"\tfps: %0.2f (interval: %lld)\n"
+			"\tformat: %s",
+			obs_source_get_name(source),
+			(const char*)name_utf8,
+			(const char*)path_utf8,
+			videoConfig.cx, videoConfig.cy,
+			fps, videoConfig.frameInterval,
+			formatName->array);
+
 	SetupBuffering(settings);
 
 	return true;
@@ -831,7 +909,38 @@ bool DShowInput::UpdateAudioConfig(obs_data_t *settings)
 			placeholders::_3, placeholders::_4,
 			placeholders::_5);
 
-	return device.SetAudioConfig(&audioConfig);
+	audioConfig.mode =
+		(AudioMode)obs_data_get_int(settings, AUDIO_OUTPUT_MODE);
+
+	bool success = device.SetAudioConfig(&audioConfig);
+	if (!success)
+		return false;
+
+	BPtr<char> name_utf8;
+	os_wcs_to_utf8_ptr(audioConfig.name.c_str(), audioConfig.name.size(),
+			&name_utf8);
+
+	blog(LOG_INFO, "\tusing video device audio: %s",
+			audioConfig.useVideoDevice ? "yes" : "no");
+
+	if (!audioConfig.useVideoDevice)
+		blog(LOG_INFO, "\taudio device: %s", (const char*)name_utf8);
+
+	const char *mode = "";
+
+	switch (audioConfig.mode) {
+		case AudioMode::Capture:     mode = "Capture"; break;
+		case AudioMode::DirectSound: mode = "DirectSound"; break;
+		case AudioMode::WaveOut:     mode = "WaveOut"; break;
+	}
+
+	blog(LOG_INFO, "\tsample rate: %d\n"
+			"\tchannels: %d\n"
+			"\taudio type: %s",
+			audioConfig.sampleRate,
+			audioConfig.channels,
+			mode);
+	return true;
 }
 
 void DShowInput::SetActive(bool active_)
@@ -884,13 +993,16 @@ inline bool DShowInput::Activate(obs_data_t *settings)
 	if (!device.ConnectFilters())
 		return false;
 
+	enum video_colorspace cs = GetColorSpace(settings);
+
+	video_range_type range = GetColorRange(settings);
+	frame.full_range = range == VIDEO_RANGE_FULL;
+
 	if (device.Start() != Result::Success)
 		return false;
 
-	enum video_colorspace cs = GetColorSpace(settings);
-
 	bool success = video_format_get_parameters(
-			cs, GetColorRange(settings),
+			cs, range,
 			frame.color_matrix,
 			frame.color_range_min,
 			frame.color_range_max);
@@ -910,7 +1022,7 @@ inline void DShowInput::Deactivate()
 
 /* ------------------------------------------------------------------------- */
 
-static const char *GetDShowInputName(void)
+static const char *GetDShowInputName(void*)
 {
 	return TEXT_INPUT_NAME;
 }
@@ -951,6 +1063,8 @@ static void GetDShowDefaults(obs_data_t *settings)
 	obs_data_set_default_bool(settings, "active", true);
 	obs_data_set_default_string(settings, COLOR_SPACE, "default");
 	obs_data_set_default_string(settings, COLOR_RANGE, "partial");
+	obs_data_set_default_int(settings, AUDIO_OUTPUT_MODE,
+			(int)AudioMode::Capture);
 }
 
 struct Resolution {
@@ -1097,6 +1211,7 @@ static const VideoFormatName videoFormatNames[] = {
 	{VideoFormat::I420,  "I420"},
 	{VideoFormat::NV12,  "NV12"},
 	{VideoFormat::YV12,  "YV12"},
+	{VideoFormat::Y800,  "Y800"},
 
 	/* packed YUV formats */
 	{VideoFormat::YVYU,  "YVYU"},
@@ -1511,9 +1626,11 @@ static bool DeviceIntervalChanged(obs_properties_t *props, obs_property_t *p,
 			}
 		}
 
-		if (listed_val != val)
+		if (listed_val != val) {
 			obs_data_set_autoselect_int(settings, FRAME_INTERVAL,
 					listed_val);
+			val = listed_val;
+		}
 
 	} else {
 		obs_data_unset_autoselect_value(settings, FRAME_INTERVAL);
@@ -1638,6 +1755,8 @@ static obs_properties_t *GetDShowProperties(void *obj)
 	obs_properties_add_button(ppts, "xbar_config", TEXT_CONFIG_XBAR,
 			CrossbarConfigClicked);
 
+	obs_properties_add_bool(ppts, DEACTIVATE_WNS, TEXT_DWNS);
+
 	/* ------------------------------------- */
 	/* video settings */
 
@@ -1684,10 +1803,25 @@ static obs_properties_t *GetDShowProperties(void *obj)
 	obs_property_list_add_int(p, TEXT_BUFFERING_OFF,
 			(int64_t)BufferingType::Off);
 
+	obs_property_set_long_description(p,
+			obs_module_text("Buffering.ToolTip"));
+
+	obs_properties_add_bool(ppts, FLIP_IMAGE, TEXT_FLIP_IMAGE);
+
 	/* ------------------------------------- */
 	/* audio settings */
 
 	Device::EnumAudioDevices(data->audioDevices);
+
+	p = obs_properties_add_list(ppts, AUDIO_OUTPUT_MODE, TEXT_AUDIO_MODE,
+			OBS_COMBO_TYPE_LIST, OBS_COMBO_FORMAT_INT);
+	obs_property_list_add_int(p, TEXT_MODE_CAPTURE,
+			(int64_t)AudioMode::Capture);
+	obs_property_list_add_int(p, TEXT_MODE_DSOUND,
+			(int64_t)AudioMode::DirectSound);
+	obs_property_list_add_int(p, TEXT_MODE_WAVEOUT,
+			(int64_t)AudioMode::WaveOut);
+
 	if (!data->audioDevices.size())
 		return ppts;
 
@@ -1723,6 +1857,22 @@ void DShowModuleLogCallback(LogType type, const wchar_t *msg, void *param)
 	UNUSED_PARAMETER(param);
 }
 
+static void HideDShowInput(void *data)
+{
+	DShowInput *input = reinterpret_cast<DShowInput*>(data);
+
+	if (input->deactivateWhenNotShowing && input->active)
+		input->QueueAction(Action::Deactivate);
+}
+
+static void ShowDShowInput(void *data)
+{
+	DShowInput *input = reinterpret_cast<DShowInput*>(data);
+
+	if (input->deactivateWhenNotShowing && input->active)
+		input->QueueAction(Action::Activate);
+}
+
 void RegisterDShowSource()
 {
 	SetLogCallback(DShowModuleLogCallback, nullptr);
@@ -1732,7 +1882,10 @@ void RegisterDShowSource()
 	info.type            = OBS_SOURCE_TYPE_INPUT;
 	info.output_flags    = OBS_SOURCE_VIDEO |
 	                       OBS_SOURCE_AUDIO |
-	                       OBS_SOURCE_ASYNC;
+	                       OBS_SOURCE_ASYNC |
+	                       OBS_SOURCE_DO_NOT_DUPLICATE;
+	info.show            = ShowDShowInput;
+	info.hide            = HideDShowInput;
 	info.get_name        = GetDShowInputName;
 	info.create          = CreateDShowInput;
 	info.destroy         = DestroyDShowInput;
